/* openGL Camouflage Test */
/* by T D Riley, with help, February 2004 */

#define HIT_MAX 36		// defines the maximum number of hits before exiting program (i.e. textures x backgrounds)
#define SPHERE_SIZE 30		// defines the size of the sphere

#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
#include <math.h>
#include <iostream>
#include <sys/time.h>		/* Linux system clock functions */


//  FUNCTION DECLARATIONS
void drawText (void *, char *, float, float);
void loadNewBackground (void);
void loadNewSphere (void);
void endGame (void);
void LoadTextures (void);

int loadImage (char *);
unsigned int setTexture (void);

// GLOBAL VARIABLES


char player[20];

float posX = 0.0, posY = 0.0;
float depthSize = 0.1;
float sphereX = 0.0;
float sphereY = 0.0;
int displayMode = 0;

int alpha = 0;			// flag for viewing the alpha map/background when the spacebar is pressed
int cheat =0;			// flag set when tester cannot find target and requests "cheat mode"
int hold = 1;			// flag to determine wait screen to click when ready
int set = 0;			// flag to set which texture list to use 0=army, 1=animal, 2=own

int maxTime;			// time globals
int minTime;
double average = 0.0;

double g_fStartOfFrame_ = 0.0, g_fEndOfFrame_ = 0.0;
double myTime = 0.0;
double totalTime = 0.0;
double gameTimes[HIT_MAX][6];	// gameTimes[hitcounter][{0=time,1=background,2=texture,3=xPos,4=yPos}]
double tex_average[6][2];	// tex_average[textureList][0=texture hits, 1=texture average time]

int survivals=0;
int bg_number = 0;		// for the increments of the image backgrounds
int hitcounter = 0;		// records the number of hits so far
int texNumber = 0;		// for picking the camouflage texture

GLuint sizeX;			// the size of the image in the X direction
GLuint sizeY;			// the size of the image in the Y direction
GLubyte *data;			// A pointer to the start of the image data

unsigned char *imagedata;	// stores the image data as an array
unsigned char *alphadata;	// stores the current alpha bitmap image as an array
unsigned char *AlphaData;	// stores the current alpha array as a Stencil

unsigned int Textures[5];	// stores an array of loaded GL textures

// names of texture files
char *imageList[6] =
  { "bg_desert1.bmp", "bg_desert2.bmp", "bg_rocky.bmp", "bg_woodland1.bmp",
"bg_woodland2.bmp", "bg_woodland3.bmp" };
char *alphaList[6] =
  { "bg_desert1_alpha.bmp", "bg_desert2_alpha.bmp", "bg_rocky_alpha.bmp",
"bg_woodland1_alpha.bmp", "bg_woodland2_alpha.bmp", "bg_woodland3_alpha.bmp" };

// textureList[0=army,1=animal,2=own][textures]

char *textureList[3][6] =
  { {"sv_desert1.bmp", "sv_desert2.bmp", "sv_rocky.bmp", "sv_woodland1.bmp",
 	"sv_woodland2.bmp", "sv_woodland3.bmp"},
  	{"red.bmp", "army_Russian.bmp", "army_Bulgarian.bmp",
     "army_EastGerman.bmp", "army_Croatian.bmp", "army_Czech.bmp"},
	{"red.bmp", "animal_leopard.bmp", "animal_tiger.bmp", "animal_snake.bmp",
 	"animal_cheetah.bmp", "animal_giraffe.bmp"}
};

//************************* WAIT FUNCTION ****************************
void
wait (double seconds)
{
  struct timeval tv;
  struct timezone tz;
  double timer, old_timer;
  gettimeofday (&tv, &tz);
  old_timer = (double) tv.tv_sec + (double) tv.tv_usec / (1000 * 1000);
  timer = old_timer;
  while ((timer - old_timer) < seconds)
    {
      gettimeofday (&tv, &tz);
      timer = (double) tv.tv_sec + (double) tv.tv_usec / (1000 * 1000);
    }
}

//************************ TIMER FUCNCTIONS **************************
double
CurrentTime_ ()			/* request frame time from Linux */
{
  struct timeval tv;
  struct timezone tz;
  gettimeofday (&tv, &tz);
  return (double) tv.tv_sec + (double) tv.tv_usec / (1000 * 1000);
}

// *********************** TEXT display ********************************
void
drawText (void *font, char *text, float x, float y)
{
  glRasterPos2f (x, y);

  while (*text != '\0')
    {
      glutBitmapCharacter (font, *text);
      ++text;
    }
}

//********************** BITMAP IMAGE LOADER ****************************
int
loadImage (char *filename)
{

  FILE *file;
  GLuint size;			// size of the image in bytes.
  GLuint i;			// standard counter.
  unsigned short int planes;	// number of planes in image (must be 1)
  unsigned short int bpp;	// number of bits perglDrawPixels( sizeX, sizeY, GL_RGB, GL_UNSIGNED_BYTE, alphadata );
  //pixel (must be 24)
  char temp;			// used to convert bgr to rgb color.

  // Make sure the file exists
  if ((file = fopen (filename, "rb")) == NULL)
    {
      printf ("File Not Found : %s\n", filename);
      return 0;
    }

  // Skip to bmp header
  fseek (file, 18, SEEK_CUR);

  // read width
  if ((i = fread (&sizeX, 4, 1, file)) != 1)
    {
      printf ("Error reading width from %s.\n", filename);

      return 0;
    }
//      printf("Width of %s: %lu\n",filename, sizeX);

  //read the height
  if ((i = fread (&sizeY, 4, 1, file)) != 1)
    {
      printf ("Error reading height from %s.\n", filename);
      return 0;
    }
//      printf("Height of %s: %lu\n", filename, sizeY);

  // calculate the size (assuming 24 bpp)
  size = sizeX * sizeY * 3;

  // read the planes
  if ((fread (&planes, 2, 1, file)) != 1)
    {
      printf ("Error reading planes from %s. \n", filename);
      return 0;
    }

  if (planes != 1)
    {

      printf ("Planes from %s is not 1: %u\n", filename, planes);
      return 0;
    }

  // read the bpp
  if ((i = fread (&bpp, 2, 1, file)) != 1)
    {
      printf ("Error reading bpp from %s. \n", filename);
      return 0;
    }

  if (bpp != 24)
    {
      printf ("Bpp from %s is not 24: %u\n", filename, bpp);
      return 0;
    }

  // seek past the rest of the bitmap header
  fseek (file, 24, SEEK_CUR);

  // Read the data
  data = (GLubyte *) malloc (size);
  if (data == NULL)
    {
      printf ("Error allocating memory for colour-corrected image data");
      return 0;
    }

  if ((i = fread (data, size, 1, file)) != 1)
    {
      printf ("Error reading image data from %s.\n", filename);
      return 0;
    }

  // reverse all of the colours bgr => rgb)
  for (i = 0; i < size; i += 3)
    {
      temp = data[i];
      data[i] = data[i + 2];
      data[i + 2] = temp;
    }

  // Thats all folks
  return 1;
}

//****************** LOAD TEXTURES into Textures[]   *******************
void
LoadTextures (void)
{
  for (int i = 0; i < 6; i++)
    {
      loadImage (textureList[set][i]);
      Textures[i] = setTexture ();
    }
  printf ("textures loaded\n");
}

//******************* SET TEXTURE FUNCTION *********************************
unsigned int
setTexture (void)
{
  unsigned int textureName;
  glGenTextures (1, &textureName);
  glBindTexture (GL_TEXTURE_2D, textureName);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

  glTexImage2D (GL_TEXTURE_2D, 0, GL_RGB, sizeX, sizeY, 0, GL_RGB,
		GL_UNSIGNED_BYTE, data);
  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
  return textureName;
}

//******************* KEYBOARD FUNCTIONS **************************
void
keyFunc (unsigned char key, int x, int y)
{
  switch (key)
    {
    case ' ':			// switch between background and alpha stencil plate using spacebar
      if (alpha == 0)
	  {
   	  alpha = 1;
	  cheat = 1;
        printf ("YOU HAVE CHEATED!!!!!!!!!\n");
	  }
	else
	alpha = 0;
	  break;

      // esc or q to exit program
    case 27:

    case 'e':
      endGame ();
      break;

    case 'q':
      exit (0);
      break;
    }
	loadNewBackground();
}

//***************** LOAD A NEW BACKGROUND *******************
void
loadNewBackground (void)
{
  loadImage (imageList[bg_number]);	// load background file from imageList[]
  imagedata = data;

  loadImage (alphaList[bg_number]);	// load corresponding alpha file from imageList[]
  alphadata = data;

  for (int i = 0; i < (int) sizeX * (int) sizeY; ++i)
    {
      AlphaData[i] = alphadata[i * 3];
    }
  glutPostRedisplay ();
}

//************************ POSITION NEW SPHERE ***********************************
void
loadNewSphere (void)
{
  g_fStartOfFrame_ = CurrentTime_ ();	// reset timer for new sphere

  srand ((unsigned) time (NULL));	// reinitialise randomness

  int randX = (rand () % (sizeX - 40)) + 20;	// randomly position sphere
  int randY = rand () % (sizeY / 2) + 20;
  posX = (float) randX;
  posY = (float) randY;

  depthSize = 1 - (posY / sizeY);	// resize sphere according to y position, to fake depth

  glutPostRedisplay ();
}

//*********************** DISPLAY FUNCTION *************************************
void
display (void)
{
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);	// clear the screen
	glClearColor (0.0, 0.0, 0.0, 0.0);

	if(displayMode==0)
	{						//					THIS DISPLAYS THE START SCREEN
		glLoadIdentity ();
		glColor4f (1.0, 0.0, 0.0, 1.0);
		drawText (GLUT_BITMAP_HELVETICA_18, "PLEASE READ THIS CAREFULLY:", (int) (sizeX / 2)-140, (int) (sizeY / 2)+170);
		drawText (GLUT_BITMAP_HELVETICA_18, "This test lasts 5 minutes", (int) (sizeX / 2)-100, (int) (sizeY / 2)+120);
		drawText (GLUT_BITMAP_HELVETICA_18, "Your aim is to locate and click on", (int) (sizeX / 2)-140 , (int) (sizeY / 2)+80);
		drawText (GLUT_BITMAP_HELVETICA_18, "the camouflaged sphere, hidden in each scene", (int) (sizeX / 2)-180 , (int) (sizeY / 2)+50);
		drawText (GLUT_BITMAP_HELVETICA_18, "* example", (int) (sizeX / 2)+50 , (int) (sizeY / 2)-10);
		// display arbitrary sphere
		glEnable (GL_TEXTURE_2D);
		  		glLoadIdentity ();
		glTranslatef ((sizeX / 2), (sizeY / 2), 0);
		glBindTexture (GL_TEXTURE_2D, Textures[2]);
		glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glEnable (GL_TEXTURE_GEN_S);
		glEnable (GL_TEXTURE_GEN_T);
		glutSolidSphere (20, 20, 20);
		glDisable(GL_TEXTURE_2D);

		glColor4f (1.0, 0.0, 0.0, 1.0);
		glLoadIdentity ();
		drawText (GLUT_BITMAP_HELVETICA_18, "If you can't find it PRESS SPACEBAR", (int) (sizeX / 2)-140 , (int) (sizeY / 2)-100);
		  drawText (GLUT_BITMAP_HELVETICA_18, "Please click to START", (int) (sizeX / 2)-90 , (int) (sizeY / 2)-150);
	}

	else  if (displayMode == 1)		// main game view
    {
		if (hold == 1)				// wait screen
		{

		glRasterPos2f (0.0, 0.0);
		glLoadIdentity ();
		glColor4f (1.0, 0.0, 0.0, 1.0);

		char hitsLeft[15];
		sprintf (hitsLeft, "\nto go: %d", (HIT_MAX - hitcounter));

		glutSwapBuffers ();
		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);	// clear the screen
		glLoadIdentity ();

		if(myTime<1.5&&myTime>0)
		drawText (GLUT_BITMAP_HELVETICA_18, "THE TARGET WAS DESTROYED",
				((int) (sizeX / 2) - 130), ((int) (sizeY / 2) + 50));
		else if(myTime>=1.5)
		{
		drawText (GLUT_BITMAP_HELVETICA_18, "THE TARGET SURVIVED",
				((int) (sizeX / 2) - 100), ((int) (sizeY / 2) + 50));
		survivals++;
		}

		drawText (GLUT_BITMAP_HELVETICA_18, "3", (int) (sizeX / 2),
				(int) (sizeY / 2));
		drawText (GLUT_BITMAP_HELVETICA_18, hitsLeft,
				((int) (sizeX / 2) - 30), ((int) (sizeY / 2) - 50));
		glutSwapBuffers ();
		wait (0.7);

		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);	// clear the screen
		glLoadIdentity ();

		if(myTime<1.5&&myTime>0)
		drawText (GLUT_BITMAP_HELVETICA_18, "THE TARGET WAS DESTROYED",
				((int) (sizeX / 2) - 130), ((int) (sizeY / 2) + 50));
		else if(myTime>=1.5)
		drawText (GLUT_BITMAP_HELVETICA_18, "THE TARGET SURVIVED",
				((int) (sizeX / 2) - 100), ((int) (sizeY / 2) + 50));

		drawText (GLUT_BITMAP_HELVETICA_18, "2", (int) (sizeX / 2),
				(int) (sizeY / 2));
		drawText (GLUT_BITMAP_HELVETICA_18, hitsLeft,
				((int) (sizeX / 2) - 30), ((int) (sizeY / 2) - 50));
		glutSwapBuffers ();
		wait (0.7);

		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);	// clear the screen
		glLoadIdentity ();

		if(myTime<1.5&&myTime>0)
		drawText (GLUT_BITMAP_HELVETICA_18, "THE TARGET WAS DESTROYED",
				((int) (sizeX / 2) - 130), ((int) (sizeY / 2) + 50));
		else if(myTime>=1.5)
		drawText (GLUT_BITMAP_HELVETICA_18, "THE TARGET SURVIVED",
				((int) (sizeX / 2) - 100), ((int) (sizeY / 2) + 50));

		drawText (GLUT_BITMAP_HELVETICA_18, "1", (int) (sizeX / 2),
				(int) (sizeY / 2));
		drawText (GLUT_BITMAP_HELVETICA_18, hitsLeft,
				((int) (sizeX / 2) - 30), ((int) (sizeY / 2) - 50));
		glutSwapBuffers ();
		wait (0.7);

		hold = 0;							// reset hold state
		alpha = 0;							// reset alpha
		cheat = 0;							// reset cheat mode
		loadNewBackground ();
		glutSwapBuffers ();
		loadNewSphere ();
		// quit
		}

      else if (hold == 0)	// go to game screen
	{

	  glLoadIdentity ();
	  glRasterPos2f (0.0, 0.0);
	  if (alpha == 0)
	    glDrawPixels (sizeX, sizeY, GL_RGB, GL_UNSIGNED_BYTE, imagedata);
	  else
	    glDrawPixels (sizeX, sizeY, GL_RGB, GL_UNSIGNED_BYTE, alphadata);

	  glDrawPixels (sizeX, sizeY, GL_STENCIL_INDEX, GL_UNSIGNED_BYTE,
			AlphaData);

	  // DRAW SPHERES

	  glEnable (GL_TEXTURE_2D);
	  glEnable (GL_STENCIL_TEST);
	  glEnable (GL_BLEND);
	  glBlendFunc (GL_DST_ALPHA, GL_ONE_MINUS_DST_ALPHA);

	  glPushMatrix ();
	  glLoadIdentity ();

	  glTranslatef (posX, posY, 0);	//position sphere

	  glBindTexture (GL_TEXTURE_2D, Textures[texNumber]);

	  glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	  glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	  glEnable (GL_TEXTURE_GEN_S);
	  glEnable (GL_TEXTURE_GEN_T);
	  glutSolidSphere ((SPHERE_SIZE * depthSize), 20, 20);
	}
    }

  else			// end of game screen (i.e displayMode==2)
    {
      glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);	// clear the screen
      // end of game display
      glEnable (GL_TEXTURE_2D);


      int increment = 70;

      glPushMatrix ();
      glLoadIdentity ();

      for (int i = 0; i < 6; i++)
	{
	  glLoadIdentity ();
	  glTranslatef (50, increment, 0);	//position sphere
	  glBindTexture (GL_TEXTURE_2D, Textures[i]);
	  glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	  glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	  glEnable (GL_TEXTURE_GEN_S);
	  glEnable (GL_TEXTURE_GEN_T);
	  glutSolidSphere (20, 20, 20);
	  increment += 70;
	}

      glLoadIdentity ();
      glRasterPos2f (0.0, 0.0);

      char averageBuffer[30];
      char endMessage[20];

      sprintf (endMessage, "THANKS FOR PLAYING, %s", player);
      drawText (GLUT_BITMAP_HELVETICA_18, endMessage, 170, 350);

      // load avergage, kills and survivals
      sprintf (averageBuffer, "average time: %f    %d destroyed   %d survived",
	       average, (hitcounter-survivals), survivals);
      drawText (GLUT_BITMAP_HELVETICA_18, averageBuffer, 170, 250);
      // load in max time

      drawText (GLUT_BITMAP_HELVETICA_18, "BEST camouflage", 160, 200);
		glLoadIdentity ();
		glTranslatef (370, 200, 0);	//position sphere
		glBindTexture (GL_TEXTURE_2D, Textures[maxTime]);
		glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glEnable (GL_TEXTURE_GEN_S);
		glEnable (GL_TEXTURE_GEN_T);
		glutSolidSphere (20, 20, 20);
      // load in min time

	 glLoadIdentity ();
	 glRasterPos2f (0.0, 0.0);
	 	drawText (GLUT_BITMAP_HELVETICA_18, "WORST camouflage", 160, 150);
		glTranslatef (370, 150, 0);	//position sphere
		glBindTexture (GL_TEXTURE_2D, Textures[minTime]);
		glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		glEnable (GL_TEXTURE_GEN_S);
		glEnable (GL_TEXTURE_GEN_T);
		glutSolidSphere (20, 20, 20);
      // quit


	  glRasterPos2f (0.0, 0.0);
	  glLoadIdentity ();
      drawText (GLUT_BITMAP_HELVETICA_18, "PRESS  Q  TO QUIT", 170, 50);
    }

  glDisable (GL_STENCIL_TEST);
  glDisable (GL_TEXTURE_2D);
  glDisable (GL_LIGHTING);
  glFlush ();
  glutSwapBuffers ();
}

//<<<<<<<<<<<<<<<<<<<<<<< myInit >>>>>>>>>>>>>>>>>>>>
void
init (void)
{
// load start time
  g_fStartOfFrame_ = CurrentTime_ ();

  glClearColor (0.0, 0.0, 0.0, 0.0);	// background color is black
  glShadeModel (GL_SMOOTH);	// render things smoothly
  glEnable (GL_DEPTH_TEST);	// enable depth testing for drawing
  glEnable (GL_TEXTURE_2D);	// enable texturing

  glMatrixMode (GL_PROJECTION);	// set "camera shape"
  glLoadIdentity ();
  int sX = sizeX;
  int sY = sizeY;
  glOrtho (0.0, sX, 0.0, sY, -100.0, 100.0);
//  gluOrtho2D(-sizeX,sizeX , 0.0,sizeY);
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity ();
  glStencilFunc (GL_NOTEQUAL, 0, 0xFFFFFFFF);
  glStencilOp (GL_KEEP, GL_KEEP, GL_KEEP);
}

//***************************************************************
void
idleFunc (void)
{
glutPostRedisplay();
// called every time the system is idle
}

//******************* MOUSE FUNCTIONS ******************************
void
mouse (int button, int state, int x, int y)
{
  float hitOffsetX = (SPHERE_SIZE * depthSize);
  float hitOffsetY = (SPHERE_SIZE * depthSize);
  sphereX = posX;
  sphereY = posY;
  y = sizeY - y;		// flip over the y mouse coord into modelview matrix space

if(state == GLUT_UP && displayMode == 0)
displayMode=1;

  else if (hold != 1)
    {
	if (state == GLUT_UP)
	{
	  if (x >= sphereX - hitOffsetX && x <= sphereX + hitOffsetX
	      && y >= sphereY - hitOffsetY && y <= sphereY + hitOffsetY)  // calculate if mouse is over sphere
	    {
	      g_fEndOfFrame_ = CurrentTime_ ();
	      myTime = g_fEndOfFrame_ - g_fStartOfFrame_;

	      printf ("[%.2d]  %10f %20s %20s %7d %7d      %d\n", hitcounter, myTime, imageList[bg_number], textureList[set][texNumber], (int) posX, (int) posY, cheat);				//	print out game data
	      totalTime += myTime;
	      gameTimes[hitcounter][0] = myTime;
	      gameTimes[hitcounter][1] = bg_number;
	      gameTimes[hitcounter][2] = texNumber;
	      gameTimes[hitcounter][3] = posX;
	      gameTimes[hitcounter][4] = posY;
		gameTimes[hitcounter][5] = cheat;


	      if (texNumber == 5)	// this test cycles textures
		texNumber = -1;

	      if (bg_number == 5)	// this test cycles background
		{
		  bg_number = -1;
		  texNumber++;
		}

	      bg_number++;			// load new background and load new sphere
	      texNumber++;
	      loadNewBackground ();
	      loadNewSphere ();

	      hitcounter++;

		// when all spheres have been hit, on each background, then hitcounter=HIT_MAX, therfore go to end screen
	    if (hitcounter == HIT_MAX)
		endGame ();
		// ************************

		hold=1;		// after a hit, display hold screen
		}

	  else if(displayMode==1)			// if mouse is clicked but sphere not hit, print missed
	    {
	      printf ("Missed!!!\n");
	    }
	}
    }
  glutPostRedisplay ();
}

//***************************************************************
void
endGame (void)
{
  maxTime = 0;
  minTime = 0;

// save game info to file

  FILE *file;
  char fileName[30];
  sprintf (fileName, "GAME_%s.txt", player);	// load in total time
  file = fopen (fileName, "a+");
  fprintf (file,
	   "*************************************************************************************\n");
  fprintf (file, "PLAYER NAME: %s (%f)\n", player, CurrentTime_ ());
  fprintf (file,
	   "Index     Time          Texture          Background           xPos    yPos    Cheated?\n\n");


// REORDERING ALGORITHM TO PRINT RESULTS IN ORDER OF TEXTURES AND BACKGROUNDS
  for (int i = 0; i < 6; i++)							// order textures
    {
		for(int k = 0; k < 6; k++)						// order backgrounds
		{
    	  	for (int j = 0; j < hitcounter; j++)		// loop through game data and print out ordered data
			{
			if (gameTimes[j][2] == i && gameTimes[j][1] == k)
				{

				fprintf (file, "[%.2d]  %10f %20s %20s %7d %7d      %d\n", j,
					gameTimes[j][0],
					textureList[set][(int) gameTimes[j][2]],
					imageList[(int) gameTimes[j][1]],
					(int) gameTimes[j][3], (int) gameTimes[j][4], (int)gameTimes[j][5]);
				tex_average[i][0]++;	//increment texture hits
				tex_average[i][1] += gameTimes[j][0];	//increment texture average time
	    		}

	  			if (gameTimes[i][0] > maxTime)
	    		maxTime = i;

	  			if (gameTimes[i][0] < minTime && gameTimes[i][0] > 0)
	    		minTime = i;
			}
		}
    tex_average[i][1] = tex_average[i][1] / tex_average[i][0];
    }

	average = totalTime / hitcounter;	// calculate the average time of all times
  	displayMode = 2;					// set flag to display end screen
  	glutPostRedisplay ();
}

//***************************************************************

int
main (int argc, char **argv)
{
  printf ("\nENTER NAME, no spaces\n");
  scanf ("%s", player);

//                              CHOOSE WHICH TEXTURE SET
  printf
    ("\nThankyou: %s\n\nPLEASE CHOOSE WHICH TEXTURE SET YOU WISH TO TEST\n0: USER CREATED DYNAMIC TEXTURES\n1: ARMY FABRICS\n2: ANIMAL SKINS\n\nPLEASE TYPE 0, 1 or 2\n",
     player);
  scanf ("%d", &set);
  if (set < 0)
    set = 0;
  if (set > 2)
    set = 2;

// 								INITIALISE GLUT
  glutInit (&argc, argv);
  glutInitWindowSize (640, 480);
  glutInitWindowPosition(500, 250);
  glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_STENCIL);
  glutCreateWindow ("Camouflage Test");

  LoadTextures ();							//
  loadImage (imageList[0]);					//name of the background file
  imagedata = data;
  loadImage (alphaList[0]);
  alphadata = data;
  AlphaData = new unsigned char[sizeX * sizeY];
  for (int i = 0; i < (int) sizeX * (int) sizeY; ++i)
    {
      AlphaData[i] = alphadata[i * 3];
    }

  glutKeyboardFunc (keyFunc);

  glutMouseFunc (mouse);
	glutDisplayFunc (display);

  loadNewSphere ();

  glutIdleFunc (idleFunc);
  init ();
  glutMainLoop ();

  return 1;
}
