// game screen loader

// ./screenLoader (texture) (bg_name) (bg_name_alpha) (posX) (posY)
/* by T D Riley, with help, February 2004 */

#define SPHERE_SIZE 30		// defines the size of the sphere

#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
#include <iostream>

int loadImage (char *);
unsigned int setTexture (void);

float posX = 0.0, posY = 0.0;
float depthSize = 0.1;

char *texture, *background, *alpha;

GLuint sizeX;			// the size of the image in the X direction
GLuint sizeY;			// the size of the image in the Y direction
GLubyte *data;			// A pointer to the start of the image data
GLuint tex;

unsigned char *imagedata;	// stores the image data as an array
unsigned char *alphadata;	// stores the current alpha bitmap image as an array
unsigned char *AlphaData;	// stores the current alpha array as a Stencil


//********************** BITMAP IMAGE LOADER ****************************
int
loadImage (char *filename)
{

  FILE *file;
  GLuint size;			// size of the image in bytes.
  GLuint i;			// standard counter.
  unsigned short int planes;	// number of planes in image (must be 1)
  unsigned short int bpp;	// number of bits perglDrawPixels( sizeX, sizeY, GL_RGB, GL_UNSIGNED_BYTE, alphadata );
  //pixel (must be 24)
  char temp;			// used to convert bgr to rgb color.

  // Make sure the file exists
  if ((file = fopen (filename, "rb")) == NULL)
    {
      printf ("File Not Found : %s\n", filename);
      return 0;
    }

  // Skip to bmp header
  fseek (file, 18, SEEK_CUR);

  // read width
  if ((i = fread (&sizeX, 4, 1, file)) != 1)
    {
      printf ("Error reading width from %s.\n", filename);

      return 0;
    }
//      printf("Width of %s: %lu\n",filename, sizeX);

  //read the height
  if ((i = fread (&sizeY, 4, 1, file)) != 1)
    {
      printf ("Error reading height from %s.\n", filename);
      return 0;
    }
//      printf("Height of %s: %lu\n", filename, sizeY);

  // calculate the size (assuming 24 bpp)
  size = sizeX * sizeY * 3;

  // read the planes
  if ((fread (&planes, 2, 1, file)) != 1)
    {
      printf ("Error reading planes from %s. \n", filename);
      return 0;
    }

  if (planes != 1)
    {

      printf ("Planes from %s is not 1: %u\n", filename, planes);
      return 0;
    }

  // read the bpp
  if ((i = fread (&bpp, 2, 1, file)) != 1)
    {
      printf ("Error reading bpp from %s. \n", filename);
      return 0;
    }

  if (bpp != 24)
    {
      printf ("Bpp from %s is not 24: %u\n", filename, bpp);
      return 0;
    }

  // seek past the rest of the bitmap header
  fseek (file, 24, SEEK_CUR);

  // Read the data
  data = (GLubyte *) malloc (size);
  if (data == NULL)
    {
      printf ("Error allocating memory for colour-corrected image data");
      return 0;
    }

  if ((i = fread (data, size, 1, file)) != 1)
    {
      printf ("Error reading image data from %s.\n", filename);
      return 0;
    }

  // reverse all of the colours bgr => rgb)
  for (i = 0; i < size; i += 3)
    {
      temp = data[i];
      data[i] = data[i + 2];
      data[i + 2] = temp;
    }

  // Thats all folks
  return 1;
}

//******************* SET TEXTURE FUNCTION *********************************
unsigned int
setTexture (void)
{
  unsigned int textureName;
  glGenTextures (1, &textureName);
  glBindTexture (GL_TEXTURE_2D, textureName);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

  glTexImage2D (GL_TEXTURE_2D, 0, GL_RGB, sizeX, sizeY, 0, GL_RGB,
		GL_UNSIGNED_BYTE, data);
  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
  return textureName;
}


void
display (void)
{
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);	// clear the screen
	glClearColor (0.0, 0.0, 0.0, 0.0);


	  glLoadIdentity ();
	  glRasterPos2f (0.0, 0.0);

	    glDrawPixels (sizeX, sizeY, GL_RGB, GL_UNSIGNED_BYTE, imagedata);
	  glDrawPixels (sizeX, sizeY, GL_STENCIL_INDEX, GL_UNSIGNED_BYTE,
			AlphaData);

	  // DRAW SPHERES

	  glEnable (GL_TEXTURE_2D);
	  glEnable (GL_STENCIL_TEST);
	  glEnable (GL_BLEND);
	  glBlendFunc (GL_DST_ALPHA, GL_ONE_MINUS_DST_ALPHA);

	  glPushMatrix ();
	  glLoadIdentity ();



	  glTranslatef (posX, posY, 0);	//position sphere

	  glBindTexture (GL_TEXTURE_2D, tex);

	  glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	  glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	  glEnable (GL_TEXTURE_GEN_S);
	  glEnable (GL_TEXTURE_GEN_T);
	  glutSolidSphere ((SPHERE_SIZE * depthSize), 20, 20);

	    glDisable (GL_STENCIL_TEST);
  glDisable (GL_TEXTURE_2D);
  glDisable (GL_LIGHTING);
  glFlush ();
  glutSwapBuffers ();
}

//<<<<<<<<<<<<<<<<<<<<<<< myInit >>>>>>>>>>>>>>>>>>>>
void
init (void)
{

  glClearColor (0.0, 0.0, 0.0, 0.0);	// background color is black
  glShadeModel (GL_SMOOTH);	// render things smoothly
  glEnable (GL_DEPTH_TEST);	// enable depth testing for drawing
  glEnable (GL_TEXTURE_2D);	// enable texturing

  glMatrixMode (GL_PROJECTION);	// set "camera shape"
  glLoadIdentity ();
  int sX = sizeX;
  int sY = sizeY;
  glOrtho (0.0, sX, 0.0, sY, -100.0, 100.0);
//  gluOrtho2D(-sizeX,sizeX , 0.0,sizeY);
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity ();
  glStencilFunc (GL_NOTEQUAL, 0, 0xFFFFFFFF);
  glStencilOp (GL_KEEP, GL_KEEP, GL_KEEP);
}

//***************************************************************
void
idleFunc (void)
{
// called every time the system is idle
}

int
main (int argc, char **argv)
{
// 								INITIALISE GLUT
  glutInit (&argc, argv);
  glutInitWindowSize (640, 480);
  glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_STENCIL);
  glutCreateWindow ("Game Screen Redisplay");

	if(argc < 6)
	{
	printf("Error: not enough arguments\n");
	exit (0);
	}

   texture=argv[1];
   background=argv[2];
   alpha=argv[3];
   sscanf (argv[4], "%f", &posX);
   sscanf (argv[5], "%f", &posY);

  loadImage (background);					//name of the background file
  imagedata = data;
 loadImage (alpha);
  alphadata = data;

  AlphaData = new unsigned char[sizeX * sizeY];
  for (int i = 0; i < (int) sizeX * (int) sizeY; ++i)
    {
      AlphaData[i] = alphadata[i * 3];
    }

	loadImage (texture);
tex = setTexture ();
depthSize = 1 - (posY / sizeY);

	glutDisplayFunc (display);
  glutIdleFunc (idleFunc);
  init ();
  glutMainLoop ();

  return 1;
}
