
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//																													//
// 								Camouflage Texture Maker through Image Processing 									//
// 								by Tim Riley, with help, February 2004 												//
//																													//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
#include <math.h>
#include <iostream>

GLuint sizeX;					// the size of the image in the X direction
GLuint sizeY;					// the size of the image in the Y direction
GLubyte *data;					// A pointer to the start of the image data
unsigned char *rawdata;			// character array for storing original image pixel data
unsigned char *edgydata;		// character array for storing processed image pixel data

int min, max;					// initialise detection range limits
int number = 0;					// sets index in imageList to choose backgrounds
int saved=0;					// flag to display saved message
int randomColour1 = 180,
	randomColour2 = 120,
	randomColour3 = 60;			// presets for selection of random colours from backgrounds

char *imageList[6] = { "img_desert1.bmp", "img_desert2.bmp", "img_rocky.bmp",
					   "img_woodland1.bmp", "img_woodland2.bmp", "img_woodland3.bmp" };

char *saveList[6] = { "sv_desert1.bmp", "sv_desert2.bmp", "sv_rocky.bmp",
					  "sv_woodland1.bmp", "sv_woodland2.bmp", "sv_woodland3.bmp" };

//********************  LOAD IMAGE FUNCTION, from Jon Macey  **************************
int
loadImage (char *filename)
{

  FILE *file;
  GLuint size;					// size of the image in bytes.
  GLuint i;						// standard counter.
  unsigned short int planes;	// number of planes in image (must be 1)
  unsigned short int bpp;		// number of bits per pixel (must be 24)
  char temp;					// used to convert bgr to rgb color.

  // Make sure the file exists
  if ((file = fopen (filename, "rb")) == NULL)
    {
      printf ("File Not Found : %s\n", filename);
      return 0;
    }

  // Skip to bmp header
  fseek (file, 18, SEEK_CUR);

  // read width
  if ((i = fread (&sizeX, 4, 1, file)) != 1)
    {
      printf ("Error reading width from %s.\n", filename);
      return 0;
    }
//      printf("Width of %s: %lu\n",filename, sizeX);

  //read the height
  if ((i = fread (&sizeY, 4, 1, file)) != 1)
    {
      printf ("Error reading height from %s.\n", filename);
      return 0;
    }
//      printf("Height of %s: %lu\n", filename, sizeY);

  // calculate the size (assuming 24 bpp)
  size = sizeX * sizeY * 3;

  // read the planes
  if ((fread (&planes, 2, 1, file)) != 1)
    {
      printf ("Error reading planes from %s. \n", filename);
      return 0;
    }

  if (planes != 1)
    {
      printf ("Planes from %s is not 1: %u\n", filename, planes);
      return 0;
    }

  // read the bpp
  if ((i = fread (&bpp, 2, 1, file)) != 1)
    {
      printf ("Error reading bpp from %s. \n", filename);
      return 0;
    }

  if (bpp != 24)
    {
      printf ("Bpp from %s is not 24: %u\n", filename, bpp);
      return 0;
    }

  // seek past the rest of the bitmap header
  fseek (file, 24, SEEK_CUR);

  // Read the data
  data = (GLubyte *) malloc (size);
  if (data == NULL)
    {
      printf ("Error allocating memory for colour-corrected image data");
      return 0;
    }

  if ((i = fread (data, size, 1, file)) != 1)
    {
      printf ("Error reading image data from %s.\n", filename);
      return 0;
    }

  // reverse all of the colours bgr => rgb)
  for (i = 0; i < size; i += 3)
    {
      temp = data[i];
      data[i] = data[i + 2];
      data[i + 2] = temp;
    }

  return 1;
}

//********************** SAVE BITMAP FUNCTION, written with Jon Macey **************************
int
saveImage (char *filename, char *savename)
{

  FILE *save, *file;
  GLuint size = sizeX * sizeY * 3;
  GLuint i;
  char temp;
  unsigned char buffer[54];			// stores header from original background image

// Make sure the file exists
  if ((file = fopen (filename, "rb")) == NULL)
    {
      printf ("File Not Found : %s\n", filename);
      return 0;
    }
// Make sure you can save the bitmap
  if ((save = fopen (savename, "w+b")) == NULL)
    {
      printf ("Could not save : %s\n", savename);
      return 0;
    }

// Copy header from original
  fread (buffer, 54, 1, file);
  fwrite (buffer, 54, 1, save);

// reorder bytes rgb > bgr
  for (GLuint j = 0; j < size; j += 3)
    {
      temp = edgydata[j];
      edgydata[j] = edgydata[j + 2];
      edgydata[j + 2] = temp;
    }

// write user manipulated pixel data to bitmap file
  fseek (save, 54, SEEK_CUR);
  if ((i = fwrite (edgydata, size, 1, save)) != 1)
    {
      printf ("Error saving edgy data to %s\n", savename);
      return 0;
    }

  fclose(file);
  fclose(save);

  return 1;
}

//****************** EDGY IMAGE PROCESSING FUNCTION ****************************
void
edgy (void)
{
// choose three base colours, by randomly selecting pixels
  int index1 = (sizeX * 3 * randomColour1);
  int index2 = (sizeX * 3 * randomColour2);
  int index3 = (sizeX * 3 * randomColour3);

  unsigned int counter = 0, i;

// processes image through testing colours between max and min range
  for (i = 0; i < (sizeX * sizeY); i++)
    {
	// calculate an average intensity of the rgb components at each pixel
      int averageIntensity = (rawdata[counter] + rawdata[counter + 1] + rawdata[counter + 2]) / 3;

      if (averageIntensity >= max)
	{
	// if the pixel has greater intensity than max,
	// then assign the colour located at index1
	  edgydata[counter] = rawdata[index1];
	  edgydata[counter + 1] = rawdata[index1 + 1];
	  edgydata[counter + 2] = rawdata[index1 + 2];
	}

      else if (averageIntensity < max && averageIntensity > min)
	{
	// if the pixel has less intensity than max, but gretaer than min,
	// then assign the colour located at index2
	  edgydata[counter] = rawdata[index2];
	  edgydata[counter + 1] = rawdata[index2 + 1];
	  edgydata[counter + 2] = rawdata[index2 + 1];
	}

      else
	{
	// if the pixel has less intensity than min,
	// then assign the colour located at index3
	  edgydata[counter] = rawdata[index3];
	  edgydata[counter + 1] = rawdata[index3 + 1];
	  edgydata[counter + 2] = rawdata[index3 + 2];
	}

      counter += 3;				// jumps to next rgb truple
    }
}

//************************* KEYBOARD FUNCTION ****************************
void
keyFunc (unsigned char key, int x, int y)
{
  switch (key)
    {
	// switch to next image and load,
	// number is the index of the image in imageList[]
    case '1':
      number = 0;
	  saved=0;
      break;
    case '2':
      number = 1;
	  saved=0;
      break;
    case '3':
      number = 2;
	  saved=0;
      break;
    case '4':
      number = 3;
	  saved=0;
      break;
    case '5':
      number = 4;
	  saved=0;
      break;
    case '6':
      number = 5;
	  saved=0;
      break;

      // pick other random colours
    case 'r':
      srand ((unsigned) time (NULL));
      randomColour1 = rand () % sizeY;
      randomColour2 = rand () % sizeY;
      randomColour3 = rand () % sizeY;
      saved=0;
      break;

	  // switch which random colour is assigned to which intensity range
	  // effectively cycling the colours
    case 'c':
      int temp;
      temp = randomColour1;
      randomColour1 = randomColour3;
      randomColour3 = randomColour2;
      randomColour2 = temp;
	  saved=0;
      break;

	  // increase max and min range on key >
    case '.':
      if (max < 255 && min > 0)
	{
	  max++;
	  min--;
	  saved=0;
	}
      break;

	  // decrease max and min range on key <
    case ',':
      if (max <= 255 && min >= 0 && max >= min)
	{
	  max--;
	  min++;
	  saved=0;
	}
      break;

	  // save current image to bitmap file
    case 's':
      saveImage (imageList[number], saveList[number]);
	  saved=1;
      break;

      // esc or q to exit program
    case 27:
    case 'q':
      exit (0);
      break;
    }

	// redisplay
  loadImage (imageList[number]);
  rawdata = data;
  edgy ();
  glutPostRedisplay ();
}

// ********************************** TEXT DISPLAY ********************************************
void
drawText (void *font, char *text, float x, float y)
{
  glRasterPos2f (x, y);

  while (*text != '\0')
    {
      glutBitmapCharacter (font, *text);
      ++text;
    }
}

//********************************** DISPLAY FUNCTION ****************************************
void
display (void)
{
  glClear (GL_COLOR_BUFFER_BIT);	// clear the screen
  glColor3f(1.0,1.0,1.0);			// text colour is white

  // draw help, list key controls in viewport at top of window
  glViewport (0, sizeY, (sizeX*2), 100);
  glLoadIdentity ();
  glRasterPos2f (0.0, 0.0);
 	drawText (GLUT_BITMAP_HELVETICA_18, "CHANGE TEXTURE USING THESE CONTROLS",-0.7 ,0.7 );
	drawText (GLUT_BITMAP_HELVETICA_12, "1-6    flick between images",-0.8 ,0.3 );
	drawText (GLUT_BITMAP_HELVETICA_12, "  r      picks new random colours",-0.8 ,0 );
	drawText (GLUT_BITMAP_HELVETICA_12, "  c      cycles the 3 colours",-0.8 ,-0.3 );
	drawText (GLUT_BITMAP_HELVETICA_12, "  q      quits the program",-0.8 ,-0.6 );
	drawText (GLUT_BITMAP_HELVETICA_12, ">      increase edge detection range",0.1 ,0.3 );
	drawText (GLUT_BITMAP_HELVETICA_12, "<      decrease edge detection range",0.1 ,0 );
	drawText (GLUT_BITMAP_HELVETICA_12, "s      saves the current texture",0.1 ,-0.3 );

  	glColor3f(0.0,1.0,0.0);		// draw text in green
	if(saved==1)				// shown when "s" key is pressed
	drawText (GLUT_BITMAP_HELVETICA_12, "  TEXTURE SAVED",0.1 ,-0.7 );


  // display original background image in left viewport
  glViewport (0, 0, sizeX, sizeY);
  glLoadIdentity ();
  glRasterPos2f (-1.0, -1.0);
  glDrawPixels (sizeX, sizeY, GL_RGB, GL_UNSIGNED_BYTE, data);

  // display intensity manipulated image in right viewport
  glViewport (sizeX, 0, sizeX, sizeY);
  glLoadIdentity ();
  glRasterPos2f (-1.0, -1.0);
  glDrawPixels (sizeX, sizeY, GL_RGB, GL_UNSIGNED_BYTE, edgydata);


// refresh display
  glutSwapBuffers ();
  glFlush ();
}

//*************************** GLUT INITIALISATION **************************
void
init (void)
{
  glClearColor (0.0, 0.0, 0.0, 0.0);	// background color is black
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  gluOrtho2D (-sizeX, sizeX, 0.0, sizeY);
}

void
idleFunc (void)
{
// called every time the system is idle
}

//*************************** MAIN PROGRAM **************************
int
main (int argc, char **argv)
{
// sets the edge detection intensity range to 1/3 and 2/3 respectively by default
  min = 85;
  max = 170;
// if user wants, edge detection range can be changed on command line argument
  if (argc == 3)
    {
      sscanf (argv[1], "%d", &min);
      sscanf (argv[2], "%d", &max);
    }

  loadImage (imageList[0]);		// set display to draw first background

  rawdata = data;
  edgydata = (unsigned char *) malloc (sizeX * sizeY * 3);
  edgy ();

  init ();
  glutInit (&argc, argv);
  glutInitWindowSize ((sizeX * 2), (sizeY)+120);
  glutCreateWindow ("Camouflage Texture Maker");
  glutKeyboardFunc (keyFunc);
  glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutDisplayFunc (display);
  glutIdleFunc (idleFunc);

  glutMainLoop ();

  return 1;
}
